#' Script: De-Novo Clustering
#' Author: Ilias Lagkouvardos
#'
#' Calculate beta-diversity for microbial communities
#' based on permutational mulitvariate analysis of variances (PERMANOVA) using multiple distance matrices
#' computed from phylogenetic distances between observed organisms
#'
#' Input:
#' 1. Set the path to the directory where the file is stored 
#' 2. Write the name of the mapping file that includes the samples groups
#' 4. Write the name of the distance table
#' 5. Write the number of clusters  
#'
#' Output: 
#' The script generates three graphical outputs (pdf) and one text file
#' 1. MDS and NMDS plots showing information about beta-diversity for number of clusters
#' 2. Add additional column to the mapping file for assigned cluster groups
#'
#' Concept:
#' Samples are clustered based on the distance matrix using the Ward's hierarchical clustering method
#' The number of clusters is based on the set parameter (default 3).
#' To determine similarities between samples, a multivariate analysis is applied
#' and sample distribution is illustrated by means of MDS and NMDS (non-metric) plots

##################################################################################
######             Set parameters in this section manually                  ######
##################################################################################

#' Please set the directory of the script as the working folder (e.g D:/studyname/NGS-Data/Rhea/beta-diversity/)
#' Note: the path is denoted by forward slash "/"
setwd("D:/path/to/Rhea/3.Beta-Diversity/")         #<--- CHANGE ACCORDINGLY !!!

#' Please give the file name of the normalized OTU-table without taxonomic classification
input_otu = "OTUs_Table-norm.tab"              #<--- CHANGE ACCORDINGLY !!!

#' Please give the name of the distance matrix (generated by beta-diversity)
input_distance = "distance-matrix-gunif.tab"                   #<--- CHANGE ACCORDINGLY !!!


#' Please give the name of the meta-file that contains individual sample information
input_meta = "mapping_file.tab"                #<--- CHANGE ACCORDINGLY !!!


#' Please give the number of clusters
#' Default number of clusters is 3 
cluster_number = 3                      #<--- CHANGE ACCORDINGLY !!!


######                  NO CHANGES ARE NEEDED BELOW THIS LINE               ######

##################################################################################
######                             Main Script                              ######
##################################################################################

###################       Load all required libraries     ########################

# Check if required packages are already installed, and install if missing
packages <-c("cluster","ade4","GUniFrac","phangorn","vegan") 

# Function to check whether the package is installed
InsPack <- function(pack)
{
  if ((pack %in% installed.packages()) == FALSE) {
    install.packages(pack,repos ="http://cloud.r-project.org/")
  } 
}

# Applying the installation on the list of packages
lapply(packages, InsPack)

# Make the libraries
lib <- lapply(packages, require, character.only = TRUE)

# Check if it was possible to install all required libraries
flag <- all(as.logical(lib))


###################       Read all required input files      ####################


# Load the mapping file containing individual sample information (sample names in the first column)
meta_file <- read.table (file = input_meta, check.names = FALSE, header = TRUE, dec = ".", sep = "\t", row.names = 1, comment.char = "")

# Clean table from empty lines
meta_file <- meta_file[!apply(is.na(meta_file) | meta_file=="",1,all),,drop=FALSE]

#------------------------------------------------------------------------

# Load the tab-delimited file containing the values to be analyzed (samples names in the first column)
otu_file <- read.table (file = input_otu, check.names = FALSE, header = TRUE, dec = ".", sep = "\t", row.names = 1, comment.char = "")

# Clean table from empty lines
otu_file <- otu_file[!apply(is.na(otu_file) | otu_file =="",1,all),]

# keep only those rows that appear in the mapping file
otu_file <- otu_file[,rownames(meta_file)]

#------------------------------------------------------------------------

# Load the distance file containing individual sample information (sample names in the first column)
unifract_dist <- read.table(input_distance, header=T, row.names=1, dec=".", sep="\t")

# Create the directory where all output files are saved (is named after the number of clusters set above for comparisons)
folder_name <- paste(cluster_number,"De-Novo-Clustering",sep="_")
dir.create(folder_name)

####################       Calculate beta-diversity          ###################

# Order the mapping file by sample names (ascending)
meta_file <- meta_file[order(row.names(meta_file)),,drop=FALSE]


# Generate vector with assigned group to each sample
pam <- pam(as.dist(unifract_dist), cluster_number, diss=TRUE)
data_cluster <- as.vector(pam$clustering)
# Store the medois of the clusters
medoids <- data.frame(otu_file[,pam$medoids])
colnames(medoids) <- pam$medoids

# Add column to mapping file with information about cluster group
meta_file <- cbind(data_cluster,meta_file)

# Calculate the NMDS plot (Non-metric Multidimensional Scaling plot)
meta <- metaMDS(unifract_dist,k = 2)

# Calculate the significance of variance to compare multivariate sample means (including two or more dependent variables)
adonis<-adonis2(as.dist(unifract_dist) ~ data_cluster)
permdisp <- permutest(betadisper(as.dist(unifract_dist),as.factor(data_cluster),type="median"))

# Generated MDS and NMDS plot are saved in one pdf file (named after number of set clusters)
file_name <- paste(cluster_number,"de-novo-clustering.pdf",sep="_")
pdf(paste(folder_name,"/",file_name,sep=""))

# Calculate and display MDS plot
# Groups are based on number of set clusters
s.class(cmdscale(unifract_dist, k = 2), col = rainbow(cluster_number), cpoint = 2, 
        fac=as.factor(data_cluster), grid=T,
        sub = paste("MDS plot of Microbial Profiles\nPERMDISP     p=",permdisp[["tab"]][["Pr(>F)"]][1],"\n",
                    "PERMANOVA  p=",adonis[1,5],sep="")
)

# Display NMDS plot 
# Groups are based on number of set clusters
s.class(meta$points, col = rainbow(cluster_number), cpoint = 2, 
        fac=as.factor(data_cluster), grid=T,
        sub = paste("metaNMDS plot of Microbial Profiles\nPERMDISP   p=",permdisp[["tab"]][["Pr(>F)"]][1],"\n",
                    "PERMANOVA  p=",adonis[1,5],sep="")
)

dev.off()

#################################################################################
######                        Write Output Files                           ######
#################################################################################

# Write mapping file with additional cluster variable
write.table(meta_file,paste(folder_name,"/","mapping_file.tab",sep=""),sep = "\t",col.names = NA)

# Write the medoids of the clusters
write.table(medoids,paste(folder_name,"/","medoids.tab",sep=""),sep = "\t",col.names = NA)

# Write the modified mapping file and copy in directory Serial-Group-Comparisons if existing
suppressWarnings (try(write.table(meta_file, "../5.Serial-Group-Comparisons/mapping_file.tab", sep = "\t",col.names = NA, quote = FALSE), silent =TRUE))


if(!flag) { stop("
    It was not possible to install all required R libraries properly.
                 Please check the installation of all required libraries manually.\n
                 Required libaries:cluster, clusterSim, ade4, GUniFrac, phangorn")
}

#################################################################################
######                           End of Script                             ######
#################################################################################
